# 项目自述

## 赛道: os 原理赛道-具有实验性质的微小内核

### 队伍编号：T202410291994458

### 队伍名称：流萤超级可爱

### 队伍成员：巩森 王舒渤

### 指导老师：万夕里

## 设计背景

在我们学校的操作系统教学实验中，学生通常通过高级语言（如 C++）编写模拟文件管理系统。然而，这种高度封装的设计难以让学生理解操作系统底层的实际工作机制，例如文件系统如何与磁盘交互、操作系统如何处理硬件中断等。
Xv6 确实是一个优秀的教学操作系统，但其文件系统功能较为复杂且封装过深，例如使用多级目录结构和系统调用接口，这些特性虽然功能强大，但对初学者不够直观。更重要的是，xv6 将文件管理和其他内核模块整合在一起，缺乏清晰的模块划分，并且目前应该是没有带你从 0 搭建它的课程的。
本实验的设计源于一篇 dev 上的博客<br />https://dev.to/frosnerd/writing-my-own-boot-loader-3mld<br />在它的框架设计基础上，我们对其修改并添加了文件管理。意在简化操作系统的封装，通过构建一个基于 x86 架构的简化操作系统，帮助学生直接接触底层开发，亲身体验操作系统的基本功能如何实现，文件系统的管理。让学生去理解这种一步步设计 os 的过程，去理解计算机的一个很重要的思想，层层封装！

## 更新
目前又实现了通过延时响应设计的贪吃蛇游戏，涉及范围广，做为功能拓展，增加实验趣味性。并且通过计算机提供的随机数接口，实现了随机数功能的实现。

## 开发环境

开发工具：<br />
使用 VirtualBox 运行 Linux 环境，通过 SSH 连接 VSCode 进行开发。<br />
使用 QEMU 模拟器运行操作系统，支持硬件交互调试。<br />
目标架构： x86 架构。<br />
工具链：
NASM 汇编器、i686-elf-gcc 编译器、i686-elf-ld 链接器

## 运行方式

1.在开发环境下运行没问题。<br />
进入对应分支的文件夹后（在makefile的目录下），运行 make clean 清理文件。make run 运行文件。（三个分支都是通过makefile管理）<br /> 2.可以在 linux 主机如 arch 等运行<br /> 3.也可以尝试在 Windows 运行（Windows 支持开发小 linux）<br />

## 内核功能
1.打exit函数，会中止cpu

2.打CREATE yourfilename yourfilesize会创建文件

3.打write yourfilename yourdata写入文件管理

4.打read yourfilename读取文件

5.打ls列出文件目录

6.一些简单的硬盘输入功能实现，包括字符显示，退格空格都功能实现。

7.可以用左右键控制浮标移动并修改字符。

8.打SNAKE number（难度1，2，3）可以游玩我们通过键盘中断延时响应等精心设计的贪吃蛇。

PS:可以通过 hexdump -C my_disk.img | less指令来看磁盘有没有写进去东西。

## 设计文档与实验文档链接

### 1 设计文档：

[设计文档链接点此](./操作系统设计与分析文档.pdf)<br />

### 2 实验文档:

[实验文档链接点此](./实验文档.pdf) <br />

## 目前进度与架构设计

### 启动流程

mbr.asm -> 打印加载信息 -> 设置加载内核 -> 从磁盘加载内核 -> 切换到 32 位保护模式 -> 远跳转: 初始化堆栈 -> 启动内核

### 中断机制初始化流程

(内核启动后) -> isr_install 初始化中断门 -> 加载 idt 表 -> 启用外部中断

### 文件系统初始化流程

检查硬盘是否正常(已连接且未被占用) -> 初始化每个扇区的占用状态标记 -> 初始化根目录表

### 通用中断处理初始化流程

设置中断向量表 -> 为每个中断分配一段 asm 处理代码 -> 跳转到通用处理 isr_common_stub -> 调用对应的 handler

### 键盘初始化流程

初始化键盘中断 -> 指示键盘在产生中断时调用的函数

### 字符输入流程

键盘按下 -> 触发中断回调 -> 从键盘输入数据端口获得数据 -> 根据输入修改 buffer -> 写入显存 -> 偏移自增

### 磁盘读写流程

### 创建新文件

通过 ATA 端口实现文件管理 - >指定文件名和大小 -> 写入条目记录表 -> 等待磁盘空闲 -> 指定各项写入参数 -> 等待数据准备完成 -> 写入数据 -> 释放磁盘占用

### 写入文件（类似创建新文件）

指定文件名和写入内容 -> 从条目记录表中获取要写入的扇区 -> 等待磁盘空闲 -> 指定各项写入参数 -> 等待数据准备完成 -> 写入数据 -> 释放磁盘占用

### 读取文件

指定文件名，准备好获取数据的 buffer -> 从条目记录表中获取要读取的扇区 -> 等待磁盘空闲 -> 指定各项读取参数 -> 等待数据准备完成 -> 读取数据 -> 释放磁盘占用

### 指令处理流程

创建指令 -> 分析 buffer 跳转到对应执行的指令 -> 指令具体内容

## 成果展示

![运行界面](结果展示.png)

## 参考情况

**网络资源**

- MIT 的 xv6 教学课程

- dev.to 上的博客
  https://dev.to/frosnerd/writing-my-own-boot-loader-3mld

  我们参考这个博客里的介绍，构建出了 boot 模块，cpu 模块，drivers 模块，与 kernel 模块的基础。在它的设计基础上，修改了各个模块里的内容，实现了更丰富易懂的功能与函数实现。此外，fs 模块为我们完全原创，文件管理对于 ATA 模块的实现，应当是全网首发，无任何参考，仅参考文件管理的设计思想。

**书籍**

- 《30 天自制操作系统》<br/>

此外，使用大语言模型 chatgpt-4o 进行了适当的排错和思路指导。

## 分支使用方式

os-only-asm是我们的第一个实验的分支，仅实现bios启动和内核加载简单字符<br/>
basic-kernel是我们去除了浮标控制的第二个实验的分支。

main是主分支.

